﻿using DataSharing.Province;
using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.HotlineWeb;
using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Dtos.Province.HuiJu;
using DotNetCore.CAP;
using Hotline.Share.Dtos;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using SqlSugar;
using XF.Domain.Repository;

namespace DataSharing.Host.Controllers
{
    public class HotlineWebController : BaseController
    {
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;
        private readonly ICapPublisher _capPublisher;
        private readonly IRepository<DsOrder> _dsOrderRepository;
        private readonly IRepository<DsTelCall> _dsTelCallRepository;
        private readonly IRepository<DsSendTask> _taskRepository;
        private readonly IRepository<DsSendTaskInfo> _taskInfoRepository;
        private readonly IPusherProviderService _pusherProviderService;

        public HotlineWebController(IMapper mapper,
            IMediator mediator,
            ICapPublisher capPublisher,
            IRepository<DsOrder> dsOrderRepository,
            IRepository<DsTelCall> dsTelCallRepository,
            IRepository<DsSendTask> taskRepository,
            IRepository<DsSendTaskInfo> taskInfoRepository,
             IPusherProviderService pusherProviderService
            )
        {
            _mapper = mapper;
            _mediator = mediator;
            _capPublisher = capPublisher;
            _dsOrderRepository = dsOrderRepository;
            _dsTelCallRepository = dsTelCallRepository;
            _taskRepository = taskRepository;
            _taskInfoRepository = taskInfoRepository;
            _pusherProviderService = pusherProviderService;
        }

        /// <summary>
        /// 查询推送任务
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpGet("getdssendtask")]
        [AllowAnonymous]
        public async Task<PagedDto<DsSendTask>> GetDsSendTask([FromQuery] QuerySendTaskDto dto)
        {
            //数据查询
            RefAsync<int> total = 0;
            var items = await _taskRepository.Queryable()
                .WhereIF(dto.StartTime.HasValue, p => p.CreationTime >= dto.StartTime)
                .WhereIF(dto.EndTime.HasValue, p => p.CreationTime <= dto.EndTime)
                .WhereIF(!string.IsNullOrEmpty(dto.id), p => p.Id == dto.id)
                .WhereIF(dto.type == "1", p => p.IsSuccess == true)
                .WhereIF(dto.type == "2", p => p.IsSuccess == false)
                .OrderByDescending(p => p.CreationTime)
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total, HttpContext.RequestAborted);

            return new PagedDto<DsSendTask>(total, items);
        }

        /// <summary>
        /// 查询任务推送明细
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpGet("getdssendtaskinfo")]
        [AllowAnonymous]
        public async Task<PagedDto<DsSendTaskInfo>> GetDsSendTaskInfo([FromQuery] QuerySendTaskDto dto)
        {
            //数据查询
            RefAsync<int> total = 0;
            var items = await _taskInfoRepository.Queryable()
                .WhereIF(dto.StartTime.HasValue, p => p.CreationTime >= dto.StartTime)
                .WhereIF(dto.EndTime.HasValue, p => p.CreationTime <= dto.EndTime)
                .WhereIF(!string.IsNullOrEmpty(dto.id), p => p.TaskId == dto.id)
                .WhereIF(dto.type == "1", p => p.IsSuccess == true)
                .WhereIF(dto.type == "2", p => p.IsSuccess == false)
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total, HttpContext.RequestAborted);
            return new PagedDto<DsSendTaskInfo>(total, items);
        }

        /// <summary>
        /// 根据ID从新推送
        /// </summary>
        /// <param name="id">推送任务Id</param>
        /// <returns></returns>
        [HttpGet("fromnewtasksend")]
        [AllowAnonymous]
        public async Task FromNewTaskSend(string id)
        {
            var dto = await _taskRepository.GetAsync(p => p.Id == id, HttpContext.RequestAborted);
            await _pusherProviderService.SendProvinceDataPusher(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 根据ID从新推送
        /// </summary>
        /// <param name="id">推送任务Id</param>
        /// <returns></returns>
        [HttpGet("updatetasksend")]
        [AllowAnonymous]
        public async Task UpdateTaskSend(string id)
        {
            var dto = await _taskRepository.GetAsync(p => p.Id == id, HttpContext.RequestAborted);
            var dataOrder = System.Text.Json.JsonSerializer.Deserialize<SendRequestSubmitCaseInfo>(dto.Request, ProvinceJsonSerializerOptions.UnStandardDatetimeJsonOptions);
            if (dataOrder != null)
            {
                if (dataOrder.Data != null && dataOrder.Data.Items != null && dataOrder.Data.Items.Count > 0)
                {
                    var data = dataOrder.Data.Items[0].ToString();
                    ///ProvinceJsonSerializerOptions.UnStandardDatetimeJsonOptions
                    var temp = JsonConvert.DeserializeObject<SubmitCaseInfo>(data);

                    var ttt = System.Text.Json.JsonSerializer.Deserialize<SubmitCaseInfo>(data, ProvinceJsonSerializerOptions.UnStandardDatetimeJsonOptions);
                }
            }
            //await _pusherProviderService.SendProvinceDataPusher(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 修改状态为已推送，防止错误数据一直请求
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("updatetasksendstate")]
        [AllowAnonymous]
        public async Task UpdateTaskSendState(string id)
        {
            var dto = await _taskRepository.GetAsync(p => p.Id == id, HttpContext.RequestAborted);
            if (dto != null)
            {
                dto.IsSuccess = true;
                await _taskRepository.UpdateAsync(dto, HttpContext.RequestAborted);
            }
        }

        /// <summary>
        /// 查询工单及时上传列表
        /// </summary>
        /// <param name="StartDate"></param>
        /// <param name="EndDate"></param>
        /// <param name="Type">查询类型：1查及时上传，其他查未及时</param>
        /// <returns></returns>
        [HttpGet("getorderlist")]
        [AllowAnonymous]
        public async Task<PagedDto<DsOrder>> GetOrderlist([FromQuery] QuerySendTaskDto dto)
        {
            //数据查询
            RefAsync<int> total = 0;
            var items = await _dsOrderRepository.Queryable()
                .Where(p => p.CreationTime >= dto.StartTime && p.CreationTime <= dto.EndTime)
                .Where(p => p.FirstSendProvinceTime != null)
                .Where(p => p.StartTime != null)
                .Select(p => new
                {
                    p.OrderId,
                    p.OrderNo,
                    p.ProvinceNo,
                    p.WorkflowId,
                    p.ExternalId,
                    p.Password,
                    p.Title,
                    p.FromName,
                    p.Contact,
                    p.Content,
                    p.CaseDate,
                    p.StartTime,
                    p.CallId,
                    p.FirstSendProvinceTime,
                    p.LastSendProvinceTime,
                    p.CaseSource,
                    p.CaseSourceCode,
                    p.HotspotId,
                    p.HotspotName,
                    p.HotspotSpliceName,
                    p.ExpiredTime,
                    p.CaseType,
                    p.CaseTypeCode
                })
                .MergeTable()
                .Select(p => new
                {
                    p.OrderId,
                    p.OrderNo,
                    p.ProvinceNo,
                    p.WorkflowId,
                    p.ExternalId,
                    p.Password,
                    p.Title,
                    p.FromName,
                    p.Contact,
                    p.Content,
                    p.CaseDate,
                    p.StartTime,
                    p.CallId,
                    p.FirstSendProvinceTime,
                    p.LastSendProvinceTime,
                    p.CaseSource,
                    p.CaseSourceCode,
                    p.HotspotId,
                    p.HotspotName,
                    p.HotspotSpliceName,
                    p.ExpiredTime,
                    p.CaseType,
                    p.CaseTypeCode
                })
                .WhereIF(dto.type == "1", p => SqlFunc.DateDiff(DateType.Second, p.StartTime.Value, p.FirstSendProvinceTime.Value) <= 180)
                .WhereIF(dto.type != "1", p => SqlFunc.DateDiff(DateType.Second, p.StartTime.Value, p.FirstSendProvinceTime.Value) > 180)
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total, HttpContext.RequestAborted);
            return new PagedDto<DsOrder>(total, _mapper.Map<IReadOnlyList<DsOrder>>(items));
        }

        /// <summary>
        /// 查询通话记录及时上传列表
        /// </summary>
        /// <param name="StartDate"></param>
        /// <param name="EndDate"></param>
        /// <param name="Type">查询类型：1查及时上传，其他查未及时</param>
        /// <returns></returns>
        [HttpGet("getcalllist")]
        [AllowAnonymous]
        public async Task<PagedDto<DsTelCall>> GetCalllist([FromQuery] QuerySendTaskDto dto)
        {
            RefAsync<int> total = 0;
            var items = await _dsTelCallRepository.Queryable()
                .Where(p => p.CreationTime >= dto.StartTime && p.CreationTime <= dto.EndTime)
                .Where(p => p.FirstSendProvinceTime != null)
                .Where(p => p.OverTime != null)
                .Select(p => new
                {
                    p.CallId,
                    p.CPN,
                    p.CDPN,
                    p.CallDirection,
                    p.TelNo,
                    p.UserName,
                    p.BeginIvrTime,
                    p.EndIvrTime,
                    p.BeginQueueTime,
                    p.EndQueueTime,
                    p.BeginRingTime,
                    p.EndRingTimg,
                    p.AnsweredTime,
                    p.OverTime,
                    p.Duration,
                    p.RingTimes,
                    p.QueueTims,
                    p.OnState,
                    p.RecordingFileUrl,
                    p.ProvinceNo,
                    p.FirstSendProvinceTime,
                    p.LastSendProvinceTime,
                    p.TaskId

                })
                .MergeTable()
                .Select(p => new
                {
                    p.CallId,
                    p.CPN,
                    p.CDPN,
                    p.CallDirection,
                    p.TelNo,
                    p.UserName,
                    p.BeginIvrTime,
                    p.EndIvrTime,
                    p.BeginQueueTime,
                    p.EndQueueTime,
                    p.BeginRingTime,
                    p.EndRingTimg,
                    p.AnsweredTime,
                    p.OverTime,
                    p.Duration,
                    p.RingTimes,
                    p.QueueTims,
                    p.OnState,
                    p.RecordingFileUrl,
                    p.ProvinceNo,
                    p.FirstSendProvinceTime,
                    p.LastSendProvinceTime,
                    p.TaskId

                })
                .WhereIF(dto.type == "1", p => SqlFunc.DateDiff(DateType.Second, p.OverTime.Value, p.FirstSendProvinceTime.Value) <= 180)
                .WhereIF(dto.type != "1", p => SqlFunc.DateDiff(DateType.Second, p.OverTime.Value, p.FirstSendProvinceTime.Value) > 180)
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total, HttpContext.RequestAborted);
            return new PagedDto<DsTelCall>(total, _mapper.Map<IReadOnlyList<DsTelCall>>(items));

        }

        /// <summary>
        /// 查询没匹配到通话记录的电话来源工单
        /// </summary>
        /// <param name="StartDate"></param>
        /// <param name="EndDate"></param>
        /// <returns></returns>
        [HttpGet("getordermatchingcall")]
        [AllowAnonymous]
        public async Task<List<DsOrder>> GetOrderMatchingCall(DateTime StartDate, DateTime EndDate)
        {
            EndDate = EndDate.AddDays(1).AddSeconds(-1);

            var items = await _dsOrderRepository.Queryable()
                .LeftJoin<DsTelCall>((p, o) => p.ProvinceNo == o.ProvinceNo)
                .Where((p, o) => p.CreationTime >= StartDate && p.CreationTime <= EndDate)
                .Where((p, o) => p.CaseSourceCode == "RGDH")
                .Where((p, o) => p.FirstSendProvinceTime.HasValue)
                .Where((p, o) => o.ProvinceNo == null)
                .ToListAsync();

            return items;
        }

        /// <summary>
        /// 及时率查询
        /// </summary>
        /// <param name="StartDate"></param>
        /// <param name="EndDate"></param>
        /// <returns></returns>
        [HttpGet("calculateuploadrate")]
        [AllowAnonymous]
        public async Task<CalculateUploadRateDto> CalculateUploadRate(DateTime StartDate, DateTime EndDate)
        {
            //工单
            var orderRate = await _dsOrderRepository.Queryable()
                   .LeftJoin<DsTelCall>((p, o) => p.ProvinceNo == o.ProvinceNo)
                    .Where((p, o) => p.CreationTime >= StartDate && p.CreationTime <= EndDate)
                    .Where((p, o) => p.FirstSendProvinceTime.HasValue)
                    .Where((p, o) => p.StartTime.HasValue)
                    .Select((p, o) => new
                    {
                        p.OrderId,
                        p.StartTime,
                        p.FirstSendProvinceTime,
                        p.CaseSourceCode,
                        o.ProvinceNo
                    }).MergeTable()
                    .Select(p => new CalculateUploadRateDto
                    {
                        OrderCountNum = SqlFunc.AggregateSum(SqlFunc.IIF(p.OrderId != null, 1, 0)),//总工单数
                        OrderTimely = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.DateDiff(DateType.Second, p.StartTime.Value, p.FirstSendProvinceTime.Value) <= 180, 1, 0)),//及时
                        OrderNotTimely = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.DateDiff(DateType.Second, p.StartTime.Value, p.FirstSendProvinceTime.Value) > 180, 1, 0)),//为及时

                        RGDHOrderCount = SqlFunc.AggregateSum(SqlFunc.IIF(p.CaseSourceCode == "RGDH", 1, 0)),
                        ContainTel = SqlFunc.AggregateSum(SqlFunc.IIF(p.CaseSourceCode == "RGDH" && p.ProvinceNo != null, 1, 0)),//匹配数
                        NotContainTel = SqlFunc.AggregateSum(SqlFunc.IIF(p.CaseSourceCode == "RGDH" && p.ProvinceNo == null, 1, 0)),//未匹配数
                    })
                    .ToListAsync();

            ///电话
            var callRate = await _dsTelCallRepository.Queryable()
              .Where(p => p.CreationTime >= StartDate && p.CreationTime <= EndDate)
              .Where(p => p.FirstSendProvinceTime.HasValue)
              .Where(p => p.OverTime.HasValue)
              .Select(p => new
              {
                  p.CallId,
                  p.OverTime,
                  p.FirstSendProvinceTime
              })
              .MergeTable()
               .Select(p => new CalculateUploadRateDto
               {
                   TelCountNum = SqlFunc.AggregateSum(SqlFunc.IIF(p.CallId != null, 1, 0)),//总工单数
                   TelTimely = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.DateDiff(DateType.Second, p.OverTime.Value, p.FirstSendProvinceTime.Value) <= 180, 1, 0)),//及时
                   TelNotTimely = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.DateDiff(DateType.Second, p.OverTime.Value, p.FirstSendProvinceTime.Value) > 180, 1, 0)),//为及时
               })
               .ToListAsync();

            CalculateUploadRateDto calculateUploadRateDto = new CalculateUploadRateDto();

            if (orderRate != null && orderRate.Count > 0)
            {
                var orderRateData = orderRate[0];
                calculateUploadRateDto.OrderCountNum = orderRateData.OrderCountNum;
                calculateUploadRateDto.OrderTimely = orderRateData.OrderTimely;
                calculateUploadRateDto.OrderNotTimely = orderRateData.OrderNotTimely;
                calculateUploadRateDto.RGDHOrderCount = orderRateData.RGDHOrderCount;
                calculateUploadRateDto.ContainTel = orderRateData.ContainTel;
                calculateUploadRateDto.NotContainTel = orderRateData.NotContainTel;
            }

            if (callRate != null && callRate.Count > 0)
            {
                var callRateData = callRate[0];
                calculateUploadRateDto.TelCountNum = callRateData.TelCountNum;
                calculateUploadRateDto.TelTimely = callRateData.TelTimely;
                calculateUploadRateDto.TelNotTimely = callRateData.TelNotTimely;
            }

            return calculateUploadRateDto;
        }
    }
}
